package main

import (
	"flag"
	"fmt"
	"log"
	"os"
	"sort"
	"strconv"

	gomysql "github.com/go-sql-driver/mysql"
	"gitlab.celogeek.com/photos/api/internal/photos/api"
)

var (
	mapEnv              = map[string]string{}
	mapCurrentEnv       = map[string]string{}
	listen              = fmt.Sprintf("%s:%d", getEnv("LISTEN_HOST", "[::]"), getEnvUInt("LISTEN_PORT", 8000))
	mysqlAddr           = fmt.Sprintf("%s:%d", getEnv("MYSQL_HOST", "localhost"), getEnvUInt("MYSQL_PORT", 3306))
	mysqlUser           = getEnv("MYSQL_USER", "photos")
	mysqlPassword       = getEnv("MYSQL_PASSWD", "photos")
	mysqlDatabase       = getEnv("MYSQL_DB", "photos")
	getEnvConfig        = false
	getCurrentEnvConfig = false
)

func getEnv(key, fallback string) string {
	mapEnv[key] = fallback
	mapCurrentEnv[key] = fallback
	if value, ok := os.LookupEnv(key); ok {
		mapCurrentEnv[key] = value
		return value
	}
	return fallback
}
func getEnvUInt(key string, fallback uint) uint {
	mapEnv[key] = fmt.Sprint(fallback)
	mapCurrentEnv[key] = fmt.Sprint(fallback)
	if value, ok := os.LookupEnv(key); ok {
		uvalue, err := strconv.Atoi(value)
		if err != nil {
			log.Fatal(err)
		}
		if uvalue <= 0 {
			log.Fatalf("env %s need to greater than 0", key)
		}
		mapCurrentEnv[key] = fmt.Sprint(uvalue)
		return uint(uvalue)
	}
	return fallback
}

func GetOrderedKeys(o map[string]string) []string {
	keys := make([]string, 0, len(o))
	for k := range o {
		keys = append(keys, k)
	}
	sort.Strings(keys)
	return keys
}

func main() {
	config := &api.ServiceConfig{Mysql: gomysql.NewConfig()}

	flag.StringVar(&config.Listen, "listen", listen, "Listen address")
	flag.StringVar(&config.Mysql.Addr, "mysql-addr", mysqlAddr, "Mysql addr")
	flag.StringVar(&config.Mysql.User, "mysql-user", mysqlUser, "Mysql user")
	flag.StringVar(&config.Mysql.Passwd, "mysql-password", mysqlPassword, "Mysql password")
	flag.StringVar(&config.Mysql.DBName, "mysql-database", mysqlDatabase, "Mysql database")
	flag.BoolVar(&getEnvConfig, "generate-config", false, "Generate an env config")
	flag.BoolVar(&getCurrentEnvConfig, "generate-current-config", false, "Generate an env config with current env")
	flag.Parse()

	if getEnvConfig {
		for _, k := range GetOrderedKeys(mapEnv) {
			fmt.Printf("%s=%q\n", k, mapEnv[k])
		}
		return
	}

	if getCurrentEnvConfig {
		for _, k := range GetOrderedKeys(mapCurrentEnv) {
			fmt.Printf("%s=%q\n", k, mapCurrentEnv[k])
		}
		return
	}

	p := api.New(config)

	if err := p.Run(); err != nil {
		log.Fatal(err)
	}

}
